@@ -1,4 +1,4 @@
-/* $Id: search.c 4453 2009-12-02 03:36:22Z astyanax $ */
+/* $Id: search.c 4472 2010-01-05 23:35:50Z astyanax $ */
 /**************************************************************************
  *   search.c                                                             *
  *                                                                        *
@@ -1357,7 +1357,7 @@ void update_history(filestruct **h, const char *s)
     assert(hage != NULL && hbot != NULL);
 
     /* If this string is already in the history, delete it. */
-    p = find_history(*hage, *hbot, s, (size_t)-1);
+    p = find_history(*hage, *hbot, s, strlen(s));
 
     if (p != NULL) {
 	filestruct *foo, *bar;
